/*
	Pinguino + MOD MP3
	
	Icludes Definitions for 
	-- PIC32-PINGUINO
	-- PIC32-PINGUINO-OTG
	-- PIC32-PINGUINO-MICRO
	-- There is a separate example for the 
	   MX220 board, located at olimex.com/dev
		
	This example uses a	pinguino board and uts 
	user button (BUT) to control (PLAY/PAUSE/NEXT)
	the	MOD-MP3 module
		
	The only supported file format is .mp3
	By default, this program reads files from the root DIR
	Every .mp3 file in the current dir is assigned and index,
	which depends on the number of ONLY the MP3 files in the 
	dir. 
	
	Refer to the README file if you want to:
	-- see how to change the working DIR
	
	-- (!!!)see HOW to correctly CONNECT 
		the MOD-MP3 module and Piguino
	
	If you have any questions, email
	support@olimex.com
	
	OLIMEX, May 2012
	http://www.olimex.com/dev
		
*/

#include<interrupt.c>
#include<serial.c>

#if ( defined(PIC32_PINGUINO_OTG) || defined(PIC32_PINGUINO) || defined(PIC32_PINGUINO_MICRO) )

	//Definitions for 
	// -- PIC32-PINGUINO
	// -- PIC32-PINGUINO-OTG
	// -- PIC32-PINGUINO-MICRO

	
	#define mp3_port UART2      //The UEXT is on UART2
	#define mp3_baud 9600
	#define BUTTON1INIT TRISDbits.TRISD0 = 1
	
	#if defined(PIC32_PINGUINO_MICRO)
		#define BUTTON1 41
	#else
		#define BUTTON1 2
	#endif
	
	//YELLOW LED is D1
	#define YLEDINIT   TRISDCLR = 0x02;
	#define YLED1      PORTDSET = 0x02;
	#define YLED0      PORTDCLR = 0x02;
	#define YLEDSWITCH PORTD   ^= 0x02;
	
	//GREEN LED is B15
	#define GLEDINIT  	TRISGCLR = 0x40;
	#define GLED1 		PORTGSET = 0x40;
	#define GLED0 		PORTGCLR = 0x40;
	#define GLEDSWITCH  PORTG   ^= 0x40;
	
	#define YELLOW 0
	#define GREEN  1
	#define BOTH   2

#endif

//Blinks an LED five times - mode is YELLOW, GREEN or BOTH
	void blinkled(unsigned char mode);
	
/*	    Interrupt routine definition  */

	// Interrupt routine checks if a string has been received
	// from the module and prints it on the terminal
	// Connect the wrapper to vector 4 (Timer1 overflow)
	void ISR_wrapper_vector_4(void) __attribute__ ((section (".vector_4")));

	// The wrapper calls the "Tmr1Interrupt" function
	// each time an interrupt occurs
	void ISR_wrapper_vector_4(void) { Tmr1Interrupt(); }

	//We declare the function as an interrupt routine
	void Tmr1Interrupt(void) __attribute__ ((interrupt));
	
	// configure timer 1
	void init_timer1(void)
	{
		IntConfigureSystem(INT_SYSTEM_CONFIG_MULT_VECTOR);	// interrupt mode (interrupt.c)
		T1CON=0;			// reset timer 1 configuration
		TMR1=0;				// reset timer 1 counter register
		PR1=0x9896;			// define the period (an interrupt will occur ~ 16 times a second)
		IPC1SET=0x7;		// select interrupt priority and sub-priority
		IFS0CLR=0x10;		// clear interrupt flag
		IEC0SET=0x10;		// enable timer 1 interrupt
		T1CONCLR=0x70;	  	// set prescaler to 1:1
		T1CONSET=0x60;		// set prescaler to 1:64
		T1CONSET=0x8000; 	// turn TIMER1 ON
	}
	
	
	volatile unsigned char p_count = 0;
	volatile unsigned char was_pressed = 0;
	volatile unsigned char is_playing = 1;
	
	// The interrupt routine code			
	void Tmr1Interrupt(void)
	{
		T1CONCLR = (1<<15);  //Stop Timer 1
		
		if(!digitalRead(BUTTON1))
		{
			p_count = p_count + 1;
			was_pressed=1;
		}
		else
		{
			if(was_pressed && (p_count<18) )
			{
				GLEDSWITCH;
				//do something if pressed for a short time
				
				SerialPrintf(mp3_port, "pnum 0\r\n");
				
				p_count = 0;
				was_pressed = 0;
			}
			else 
				if(was_pressed && (p_count>=18) )
				{
					YLEDSWITCH;
					//do something if pressed for a long time
					SerialPrintf(mp3_port, "C\r\n");
					
					p_count = 0;
					was_pressed = 0;
					
				}		
		}
		//turn Timer 1 on again
		init_timer1();
	}
	
	void setup(){
		
		/* Init the Module, LEDs and Button on the pinguino board*/
		YLEDINIT;
		GLEDINIT;	
		YLED0;
		GLED0;
		BUTTON1INIT;
	
	// Open a serial communication from the board to the module
		SerialConfigure(mp3_port, UART_ENABLE, UART_RX_TX_ENABLED, mp3_baud);
		
		//while(digitalRead(BUTTON1));		// wait for a button press, uncomment if needed
	
	// This is where you make the do something for the
	// first time; make sure you have connected everyhting right
	
	// Check the README file if you want to know what messages you need to
	// send here in order to change the dir, etc.
		
		//SerialPrintf(mp3_port, "pnum 0\r\n");
		
		init_timer1();
	
	}
	
	void loop(){
		
	}
	
	void blinkled(unsigned char mode)
	{	


	//Turn off all LEDs
				YLED0;
				GLED0;
				
				char i;
				
				// Blink
				switch(mode)
				{
					case YELLOW:
						for(i=0; i<10;i++)
						{
							YLEDSWITCH;
							delay(100);
						}
						break;
					
					case GREEN:
						for(i=0; i<10;i++)
						{
							GLEDSWITCH;
							delay(100);
						}
						break;
					
					case BOTH:
						for(i=0; i<10;i++)
						{
							YLEDSWITCH;
							GLEDSWITCH;
							delay(100);
						}
						break;
				}	
	
					
					
	}